--
-- Specialization for Lowboy
--
-- @author  Xentro (http://www.ls-uk.info)
-- @date  16/07/11
--
-- Do not edit without my permission

Lowboy = {};

function Lowboy.prerequisitesPresent(specializations)
	Vehicle.registerJointType("Goosneck");
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
	--return true;
end;

function Lowboy:load(xmlFile)

	print("Lowboy.lua (trailer) have been loaded" );
	
	local rotationNode1 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.axlelift#index"));
    if rotationNode1 ~= nil then
        self.axlelift = {};
        self.axlelift.node = rotationNode1;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.axlelift#minRot"));
        self.axlelift.minRot = {};
        self.axlelift.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.axlelift.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.axlelift.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.axlelift#maxRot"));
        self.axlelift.maxRot = {};
        self.axlelift.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.axlelift.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.axlelift.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.axlelift.moveTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.axlelift#moveTime"), 2)*1000;
        self.axlelift.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.axlelift#touchRotLimit"), 10));
    end;
	
	local rotationNode2 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.axlelift2#index"));
    if rotationNode2 ~= nil then
        self.axlelift2 = {};
        self.axlelift2.node = rotationNode2;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.axlelift2#minRot"));
        self.axlelift2.minRot = {};
        self.axlelift2.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.axlelift2.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.axlelift2.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.axlelift2#maxRot"));
        self.axlelift2.maxRot = {};
        self.axlelift2.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.axlelift2.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.axlelift2.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.axlelift2.moveTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.axlelift2#moveTime"), 2)*1000;
        self.axlelift2.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.axlelift2#touchRotLimit"), 10));
    end;
	
	local airSound = getXMLString(xmlFile, "vehicle.airSound#file");
    if airSound ~= nil and airSound ~= "" then
        airSound = Utils.getFilename(airSound, self.baseDirectory);
        self.airSound = createSample("airSound");
        loadSample(self.airSound, airSound, false);
        self.airSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.airSound#pitchOffset"), 0); 
        self.airSoundEnabled = false; 
		self.airSoundSample = false;
    end;
	local airCompSound = getXMLString(xmlFile, "vehicle.airCompSound#file");
    if airCompSound ~= nil and airCompSound ~= "" then
        airCompSound = Utils.getFilename(airCompSound, self.baseDirectory);
        self.airCompSound = createSample("airCompSound");
        loadSample(self.airCompSound, airCompSound, false);
        self.airCompSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.airCompSound#pitchOffset"), 0); 
        self.airCompSoundEnabled = false; 
		self.airCompSoundSample = false;
    end;
	
	self.reverseLights = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.reverseLights#index"));	
	setVisibility(self.reverseLights, false);
	
	self.helpHudPosX = 0.833;
    self.helpHudPosY = 0.1725;
    self.helpHudWidth = 0.17;
    self.helpHudHeight = 0.55;
    self.helpHudImg = Utils.getFilename("textures/Info_hud_lowboy.png", self.baseDirectory);
	self.helpHudOverlay = Overlay:new("helpHudOverlay", self.helpHudImg, self.helpHudPosX, self.helpHudPosY, self.helpHudWidth, self.helpHudHeight);

	self.greenButton4PosX		= 0.725; 	-- button 4
    self.greenButton4PosY 		= 0.097; 	-- button 4
    self.greenButtonWidth		= 0.018;
    self.greenButtonHeight 		= 0.025;
    self.greenButtonImg = Utils.getFilename("textures/green_button.png", self.baseDirectory);
	self.greenButton4Overlay = Overlay:new("greenButton4Overlay", self.greenButtonImg, self.greenButton4PosX, self.greenButton4PosY, self.greenButtonWidth, self.greenButtonHeight);

	self.redButton4PosX			= 0.725; 	-- button 4
    self.redButton4PosY 		= 0.097; 	-- button 4
    self.redButtonWidth			= 0.018;
    self.redButtonHeight 		= 0.025;
    self.redButtonImg = Utils.getFilename("textures/red_button.png", self.baseDirectory);
	self.redButton4Overlay = Overlay:new("redButton4Overlay", self.redButtonImg, self.redButton4PosX, self.redButton4PosY, self.redButtonWidth, self.redButtonHeight);
	
	self.mesPosX				= 0.613;
    self.mesPosY 				= 0;
    self.mesWidth				= 0.112;
    self.mesHeight 				= 0.1215;
    self.mesImg = Utils.getFilename("textures/text_mes_lowboy.png", self.baseDirectory);
	self.mesOverlay = Overlay:new("mesOverlay", self.mesImg, self.mesPosX, self.mesPosY, self.mesWidth, self.mesHeight);
	
    self.helpHud 						= false;
	self.axleliftMax					= false;
	self.axleliftMax2 					= false;
	self.lowerMaxTimer 					= 5000;
	self.lowerTimer 					= self.lowerMaxTimer;
	self.hudButton 						= false;
	
	self.vehicleType = getXMLString(xmlFile, "vehicle#type");
end;

function Lowboy:delete()
	if self.airSoundEnabled  then
		stopSample(self.airSoundId);
		self.airSoundEnabled = false;
	end;
	if self.airCompSoundEnabled  then
		stopSample(self.airCompSoundId);
		self.airCompSoundEnabled = false;
	end;
end;

function Lowboy:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Lowboy:keyEvent(unicode, sym, modifier, isDown)
end;

function Lowboy:update(dt)
if self:getIsActive() then 
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.AXLE_LIFT_LOWER) then
			self.axleliftMax = not self.axleliftMax;
			self.axleliftMax2 = not self.axleliftMax2;
		end;
		
		if InputBinding.hasEvent(InputBinding.HELP_HUD2) then
			self.helpHud = not self.helpHud;
		end;
	end;  
	
	if self.movingDirection == -1 and self:doCheckSpeedLimit() and self.lastSpeed*3600 > 0.27 then
		setVisibility(self.reverseLights, true);
	elseif self.movingDirection ~=-1  then 
		setVisibility(self.reverseLights, false);
	end;

	local x, y, z = getRotation(self.axlelift.node);
	if self.axleliftMax then
		if x< Utils.degToRad(-17) then
			self.airCompSoundSample = false;
		elseif x> Utils.degToRad(-1) then
			self.airCompSoundSample = false;
		else
			self.airCompSoundSample = true;
		end;
	elseif not self.axleliftMax then
		if x< Utils.degToRad(-17) then
			self.airSoundSample = false;
		elseif x> Utils.degToRad(-1) then
			self.airSoundSample = false;
		else
			self.airSoundSample = true;
		end;
	end;
	if x< Utils.degToRad(-17) then
		self.hudButton = true;
		self.lowerTimer = self.lowerTimer - dt;
		if self.lowerTimer < 0 then
			self.axleliftMax	= false;
			self.axleliftMax2	= false;
		end;
	else
		self.lowerTimer = self.lowerMaxTimer;
		self.hudButton = false;
	end;
	
	if self.axleliftMax ~= nil then
        local x, y, z = getRotation(self.axlelift.node);
        local rot = {x,y,z};
        local newRot = Utils.getMovedLimitedValues(rot, self.axlelift.maxRot, self.axlelift.minRot, 3, self.axlelift.moveTime, dt, not self.axleliftMax);
        setRotation(self.axlelift.node, unpack(newRot));
    end;
	
	if self.axleliftMax2 ~= nil then
        local x, y, z = getRotation(self.axlelift2.node);
        local rot = {x,y,z};
        local newRot = Utils.getMovedLimitedValues(rot, self.axlelift2.maxRot, self.axlelift2.minRot, 3, self.axlelift2.moveTime, dt, not self.bottomPart2Max);
        setRotation(self.axlelift2.node, unpack(newRot));
    end;
	
	local joint = self.componentJoints[3];
	setJointFrame(joint.jointIndex, 0, joint.jointNode);
	local joint = self.componentJoints[4];
	setJointFrame(joint.jointIndex, 0, joint.jointNode);
	local joint = self.componentJoints[5];
	setJointFrame(joint.jointIndex, 0, joint.jointNode);
	
	if self.airSoundSample then
		if	not self.airSoundEnabled then
			playSample(self.airSound, 0, 1, 0);
			setSamplePitch(self.airSound, self.airSoundPitchOffset);
			self.airSoundEnabled = true;
		end;
	elseif  not self.airSoundSample then	
		if self.airSoundEnabled then
			stopSample(self.airSound);
			self.airSoundEnabled = false;
		end;
	end;
	if self.airCompSoundSample then
		if	not self.airCompSoundEnabled then
			playSample(self.airCompSound, 0, 1, 0);
			setSamplePitch(self.airCompSound, self.airCompSoundPitchOffset);
			self.airCompSoundEnabled = true;
		end;
	elseif  not self.airCompSoundSample then	
		if self.airCompSoundEnabled then
			stopSample(self.airCompSound);
			self.airCompSoundEnabled = false;
		end;
	end;
end;
end;

function Lowboy:draw()
	if not self.helpHud then
		g_currentMission:addHelpButtonText(g_i18n:getText("Help_on"), InputBinding.HELP_HUD2);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("Help_off"), InputBinding.HELP_HUD2);
		self.helpHudOverlay:render();
	end;
	
	if self.axleliftMax then
		self.greenButton4Overlay:render();
	else
		self.redButton4Overlay:render();
	end;
	self.mesOverlay:render();
end;

function Lowboy:onAttach()
--	if (attacherVehicle.aGoosneck ~= nil and attacherVehicle.aGoosneck == 88) then
--		self.attacherVehicle:detachImplementByObject(self);
--	end;
end;

function Lowboy:onDetach()
	self.helpHud = false;
	setVisibility(self.reverseLights, false);
	if self.airSoundEnabled  then
		stopSample(self.airSoundId);
		self.airSoundEnabled = false;
	end;
	if self.airCompSoundEnabled  then
		stopSample(self.airCompSoundId);
		self.airCompSoundEnabled = false;
	end;
end;

function Lowboy:onLeave()
	setVisibility(self.reverseLights, false);
	if self.airSoundEnabled  then
		stopSample(self.airSoundId);
		self.airSoundEnabled = false;
	end;
	if self.airCompSoundEnabled  then
		stopSample(self.airCompSoundId);
		self.airCompSoundEnabled = false;
	end;
end;

function Lowboy:readStream(streamId, connection)
end;

function Lowboy:writeStream(streamId, connection)
end;